/* deft.c
 * Xplico System dispatcher module for DEFT Linux distribution
 *
 *
 * Xplico - Internet Traffic Decoder
 * By Gianluca Costa <g.costa@iserm.com>
 * Copyright 2007 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <fcntl.h>
#include <sqlite.h> /* CakePHP is only compatible  with sqlite2 */

#include "proto.h"
#include "log.h"
#include "pei.h"
#include "dmemory.h"
#include "deft.h"


/* pol */
static int pol_id;
static int pol_polid_id;
static int pol_sesid_id;

/* pop id */
static int pop_id;
static int pei_pop_user_id;
static int pei_pop_pswd_id;
static int pei_pop_eml_id;

/* smtp id */
static int smtp_id;
static int pei_smtp_to_id;
static int pei_smtp_from_id;
static int pei_smtp_eml_id;

/* imap id */
static int imap_id;
static int pei_imap_user_id;
static int pei_imap_pswd_id;
static int pei_imap_eml_id;

/* http id */
static int http_id;
static int pei_http_url_id;
static int pei_http_client_id;
static int pei_http_host_id;
static int pei_http_content_type_id;
static int pei_http_method_id;
static int pei_http_status_id;
static int pei_http_req_header_id;
static int pei_http_req_body_id;
static int pei_http_res_header_id;
static int pei_http_res_body_id;

/* sqlite */
static sqlite *db;              /* sqlite DB */
static pthread_mutex_t db_mux;  /* mutex to access db */


static void DispFlowInfo(char *fname, const pstack_f *stack)
{
    char *xmlog;
    int fd;

    if (stack != NULL) {
        /* xml flow */
        xmlog = ProtStackFrmXML(stack);
        
        /* save_xml_file */
        fd = open(fname, O_CREAT|O_RDWR, 0x01B6);
        if (fd != -1) {
            write(fd, xmlog, strlen(xmlog));
            close(fd);
        }
        xfree(xmlog);
    }
}


static int DispQuery(char *query)
{
    int ret;
    char *err = NULL;

    pthread_mutex_lock(&db_mux);
    sqlite_exec(db, query, 0, 0, &err);
    pthread_mutex_unlock(&db_mux);
    ret = 0;
    #warning "to be complete"

    return ret;
}


static int DispStrCnv(char *dst, char *src, int len)
{
    int i, n, dim;

    i = 0;
    n = 0;
    dim = strlen(dst);
    if (dim != 0) {
        n = dim ;
        dst[n++] = ',';
        dst[n++] = ' ';
    }
    len = len - dim;

    while (i<len && src[i] != '\0') {
        if (src[i] == '\'') {
            dst[n] = '\\';
            n++;
        }
        if (src[i] != '\r' && src[i] != '\n') {
            dst[n] = src[i];
            n++;
        }
        i++;
    }

    return 0;
}


static int DispMimeParse(char *file, char *subject, char *from, char *to)
{
    char cmd[XS_QUERY_DIM];
    char dump[XS_STR_DIM];
    FILE *fp;
    int data;

    /* mime dump */
    sprintf(dump, "%s/dump_%i_%p.txt", ProtTmpDir(), getpid(), file);
    sprintf(cmd, "%s %s > %s", XS_MIME_PARSER, file, dump);
    system(cmd);

    /* read file */
    fp = fopen(dump, "r");
    if (fp) {
        data = 0;
        while (fgets(cmd, XS_QUERY_DIM, fp) != NULL) {
            if (strncmp(cmd, XS_MIME_TO, 3) == 0) {
                data = 1;
            }
            else if (strncmp(cmd, XS_MIME_FROM, 5) == 0) {
                data = 2;
            }
            else if (strncmp(cmd, XS_MIME_SUBJECT, 8) == 0) {
                data = 3;
            }
            else {
                switch (data) {
                case 1:
                    if (to != NULL) {
                        DispStrCnv(to, cmd, XS_STR_DIM);
                    }
                    break;
                    
                case 2:
                    if (from != NULL) {
                        DispStrCnv(from, cmd, XS_STR_DIM);
                    }
                    break;
                    
                case 3:
                    DispStrCnv(subject, cmd, XS_STR_DIM);
                    break;
                    
                default:
                    break;
                }
            }

            memset(cmd, 0, XS_QUERY_DIM);
        }
        fclose(fp);
    }

    remove(dump);

    return 0;
}


static int DispPop(pei *ppei)
{
    pei_component *cmpn;
    char query[XS_QUERY_DIM];
    char rep[XS_QUERY_DIM];
    char subject[XS_STR_DIM];
    char from[XS_STR_DIM];
    char to[XS_STR_DIM];
    char flow_info[XS_QUERY_DIM];
    int pol;
    int sess;
    char *user;
    char *pwd;
    char *path;
    struct stat info;
    const pstack_f* frame;
    ftval val;
    char *name;

    frame = ProtStackSearchProt(ppei->stack, pol_id);
    if (frame) {
        ProtGetAttr(frame, pol_polid_id, &val);
        pol = val.int32;
        ProtGetAttr(frame, pol_sesid_id, &val);
        sess = val.int32;
    }
    else {
        sess = pol = 0;
    }
    path = pwd = user = NULL;
    cmpn = ppei->components;
    while (cmpn != NULL) {
        if (cmpn->eid == pei_pop_user_id) {
            user = cmpn->strbuf;
        }
        if (cmpn->eid == pei_pop_pswd_id) {
            pwd = cmpn->strbuf;

        }
        if (cmpn->eid == pei_pop_eml_id) {
            path = cmpn->file_path;
        }
        cmpn = cmpn->next;
    }

    /* compose query and insert record */
    if (path) {
        stat(path, &info);
        /* new path */
        name = strrchr(path, '/');
        name++;
        sprintf(rep, XS_MAIL_DIR_PATH"/%s", pol, sess, name);
        rename(path, rep);
        /* flow info */
        sprintf(flow_info, XS_MAIL_DIR_PATH"/flow_%s.xml", pol, sess, name);
        DispFlowInfo(flow_info, ppei->stack);
        /* parse mime */
        memset(subject, 0, XS_STR_DIM);
        memset(from, 0, XS_STR_DIM);
        memset(to, 0, XS_STR_DIM);
        DispMimeParse(rep, subject, from, to);
        /* query */
        sprintf(query, XS_QUERY_POP_TEMPLATE, sess, pol, time(NULL)-ppei->time_cap, info.st_size, flow_info,
                user, pwd, from, to, subject, rep);
        if (DispQuery(query) != 0) {
            printf("%s\n", query);
            exit(0);
        }
    }

    return 0;
}


static int DispSmtp(pei *ppei)
{
    pei_component *cmpn;
    char query[XS_QUERY_DIM];
    char rep[XS_QUERY_DIM];
    char subject[XS_STR_DIM];
    char *from;
    char to[XS_STR_DIM];
    char flow_info[XS_QUERY_DIM];
    int pol;
    int sess;
    char *path;
    struct stat info;
    const pstack_f* frame;
    ftval val;
    char *name;

    frame = ProtStackSearchProt(ppei->stack, pol_id);
    if (frame) {
        ProtGetAttr(frame, pol_polid_id, &val);
        pol = val.int32;
        ProtGetAttr(frame, pol_sesid_id, &val);
        sess = val.int32;
    }
    else {
        sess = pol = 0;
    }
    path = NULL;
    to[0] = '\0';
    cmpn = ppei->components;
    while (cmpn != NULL) {
        if (cmpn->eid == pei_smtp_to_id) {
            if (to[0] != '\0') {
                strcat(to, ", ");
            }
            strcat(to, cmpn->strbuf);
        }
        if (cmpn->eid == pei_smtp_from_id) {
            from = cmpn->strbuf;
        }
        if (cmpn->eid == pei_smtp_eml_id) {
            path = cmpn->file_path;
        }
        cmpn = cmpn->next;
    }

    /* compose query and insert record */
    if (path) {
        stat(path, &info);
        /* new path */
        name = strrchr(path, '/');
        name++;
        sprintf(rep, XS_MAIL_DIR_PATH"/%s", pol, sess, name);
        rename(path, rep);
        /* flow info */
        sprintf(flow_info, XS_MAIL_DIR_PATH"/flow_%s.xml", pol, sess, name);
        DispFlowInfo(flow_info, ppei->stack);
        /* parse mime */
        memset(subject, 0, XS_STR_DIM);
        DispMimeParse(rep, subject, NULL, NULL);
        /* query */
        sprintf(query, XS_QUERY_SMTP_TEMPLATE, sess, pol, time(NULL)-ppei->time_cap, info.st_size, flow_info,
                from, to, subject, rep);
        if (DispQuery(query) != 0) {
            printf("%s\n", query);
            exit(0);
        }
    }

    return 0;
}


static int DispImap(pei *ppei)
{
    pei_component *cmpn;
    char query[XS_QUERY_DIM];
    char rep[XS_QUERY_DIM];
    char subject[XS_STR_DIM];
    char from[XS_STR_DIM];
    char to[XS_STR_DIM];
    char flow_info[XS_QUERY_DIM];
    int pol;
    int sess;
    char *user;
    char *pwd;
    char *path;
    struct stat info;
    const pstack_f* frame;
    ftval val;
    char *name;

    frame = ProtStackSearchProt(ppei->stack, pol_id);
    if (frame) {
        ProtGetAttr(frame, pol_polid_id, &val);
        pol = val.int32;
        ProtGetAttr(frame, pol_sesid_id, &val);
        sess = val.int32;
    }
    else {
        sess = pol = 0;
    }
    path = pwd = user = NULL;
    cmpn = ppei->components;
    while (cmpn != NULL) {
        if (cmpn->eid == pei_imap_user_id) {
            user = cmpn->strbuf;
        }
        if (cmpn->eid == pei_imap_pswd_id) {
            pwd = cmpn->strbuf;

        }
        if (cmpn->eid == pei_imap_eml_id) {
            path = cmpn->file_path;
        }
        cmpn = cmpn->next;
    }

    /* compose query and insert record */
    if (path) {
        stat(path, &info);
        /* new path */
        name = strrchr(path, '/');
        name++;
        sprintf(rep, XS_MAIL_DIR_PATH"/%s", pol, sess, name);
        rename(path, rep);
        /* flow info */
        sprintf(flow_info, XS_MAIL_DIR_PATH"/flow_%s.xml", pol, sess, name);
        DispFlowInfo(flow_info, ppei->stack);
        /* parse mime */
        memset(subject, 0, XS_STR_DIM);
        memset(from, 0, XS_STR_DIM);
        memset(to, 0, XS_STR_DIM);
        DispMimeParse(rep, subject, from, to);
        /* query */
        sprintf(query, XS_QUERY_IMAP_TEMPLATE, sess, pol, time(NULL)-ppei->time_cap, info.st_size, flow_info,
                user, pwd, from, to, subject, rep);
        if (DispQuery(query) != 0) {
            printf("%s\n", query);
            exit(0);
        }
    }

    return 0;
}


static int DispHttp(pei *ppei)
{
    pei_component *cmpn;
    char query[XS_QUERY_DIM];
    char flow_info[XS_QUERY_DIM];
    char url[XS_QUERY_DIM];
    char rq_header[XS_QUERY_DIM], rq_body[XS_QUERY_DIM], rs_header[XS_QUERY_DIM], rs_body[XS_QUERY_DIM];
    int pol;
    int sess;
    char *host, *host_t, *urlp, *path;
    char *cont_type;
    char *rep, *method, *response;
    int rq_bd_size, rs_bd_size;
    const pstack_f* frame;
    ftval val;
    char *name;

    frame = ProtStackSearchProt(ppei->stack, pol_id);
    if (frame) {
        ProtGetAttr(frame, pol_polid_id, &val);
        pol = val.int32;
        ProtGetAttr(frame, pol_sesid_id, &val);
        sess = val.int32;
    }
    else {
        sess = 0;
        pol = 0;
    }
    host = host_t = method = response = rep = cont_type = urlp = NULL;
    rq_bd_size = rs_bd_size = 0;
    memcpy(rq_header, "NULL", 5);
    memcpy(rq_body, "NULL", 5);
    memcpy(rs_header, "NULL", 5);
    memcpy(rs_body, "NULL", 5);

    cmpn = ppei->components;
    while (cmpn != NULL) {
        if (cmpn->eid == pei_http_url_id) {
            if (host_t != NULL) {
                sprintf(url, "%s%s", host_t, cmpn->strbuf);
            }
            else {
                urlp = cmpn->strbuf;
            }
        }
        else if (cmpn->eid == pei_http_host_id) {
            host = cmpn->strbuf;
            if (urlp != NULL) {
                sprintf(url, "%s%s", cmpn->strbuf, urlp);
            }
            else {
                host_t = cmpn->strbuf;
            }
        }
        else if (cmpn->eid == pei_http_content_type_id) {
            cont_type = cmpn->strbuf;
        }
        else if (cmpn->eid == pei_http_method_id) {
            method = cmpn->strbuf;
        }
        else if (cmpn->eid == pei_http_status_id) {
            response = cmpn->strbuf;
        }
        else if (cmpn->eid == pei_http_req_header_id) {
            /* new path */
            rep = rq_header;
            path = cmpn->file_path;
            name = strrchr(path, '/');
            name++;
            sprintf(rep, XS_HTTP_DIR_PATH"/%s", pol, sess, name);
            rename(path, rep);
        }
        else if (cmpn->eid == pei_http_req_body_id) {
            /* new path */
            rep = rq_body;
            path = cmpn->file_path;
            name = strrchr(path, '/');
            if (name == NULL) {
                printf("Cazzo: %s\n", path);
            }
            else {
                name++;
                sprintf(rep, XS_HTTP_DIR_PATH"/%s", pol, sess, name);
                rename(path, rep);
                rq_bd_size = cmpn->file_size;
            }
        }
        else if (cmpn->eid == pei_http_res_header_id) {
            /* new path */
            rep = rs_header;
            path = cmpn->file_path;
            name = strrchr(path, '/');
            name++;
            sprintf(rep, XS_HTTP_DIR_PATH"/%s", pol, sess, name);
            rename(path, rep);
        }
        else if (cmpn->eid == pei_http_res_body_id) {
            /* new path */
            rep = rs_body;
            path = cmpn->file_path;
            name = strrchr(path, '/');
            name++;
            sprintf(rep, XS_HTTP_DIR_PATH"/%s", pol, sess, name);
            rename(path, rep);
            rs_bd_size = cmpn->file_size;
        }

        cmpn = cmpn->next;
    }

    /* compose query and insert record */
    if (rs_header != NULL) {
        /* flow info */
        sprintf(flow_info, XS_HTTP_DIR_PATH"/flow_%s.xml", pol, sess, name);
        DispFlowInfo(flow_info, ppei->stack);
        /* query */
        sprintf(query, XS_QUERY_WEB_TEMPLATE, sess, pol, time(NULL)-ppei->time_cap, flow_info,
                url, XS_URL_REL_6, method, response, host, cont_type, rq_header, rq_body, rq_bd_size, rs_header, rs_body, rs_bd_size);
        if (DispQuery(query) != 0) {
            printf("%s\n", query);
            exit(0);
        }
    }

    return 0;
}


static int DBInit(void)
{
    char *err;

    db = sqlite_open(XS_DB_FILE, 0, &err);
    if (db == 0) {
        return -1;
    }

    return 0;
}


static int DBClose(void)
{
    sqlite_close(db);

    return 0;
}


int DispInit(void)
{
    pol_id = ProtId("pol");
    if (pol_id == -1) {
        printf("This dispacter run olny with pol dissector e capure\n");

        return -1;
    }
    pol_sesid_id = ProtAttrId(pol_id, "pol.sesid");
    pol_polid_id = ProtAttrId(pol_id, "pol.polid");

    /* pei id */
    pop_id = ProtId("pop");
    if (pop_id != -1) {
        pei_pop_user_id = ProtPeiComptId(pop_id, "user");
        pei_pop_pswd_id = ProtPeiComptId(pop_id, "password");
        pei_pop_eml_id = ProtPeiComptId(pop_id, "eml");
    }

    smtp_id = ProtId("smtp");
    if (smtp_id != -1) {
        pei_smtp_to_id = ProtPeiComptId(smtp_id, "to");
        pei_smtp_from_id = ProtPeiComptId(smtp_id, "from");
        pei_smtp_eml_id = ProtPeiComptId(smtp_id, "eml");
    }

    imap_id = ProtId("imap");
    if (imap_id != -1) {
        pei_imap_user_id = ProtPeiComptId(imap_id, "user");
        pei_imap_pswd_id = ProtPeiComptId(imap_id, "password");
        pei_imap_eml_id = ProtPeiComptId(imap_id, "eml");
    }
    
    http_id = ProtId("http");
    if (http_id != -1) {
        pei_http_url_id = ProtPeiComptId(http_id, "url");
        pei_http_client_id = ProtPeiComptId(http_id, "client");
        pei_http_host_id = ProtPeiComptId(http_id, "host");
        pei_http_content_type_id = ProtPeiComptId(http_id, "content_type");
        pei_http_method_id = ProtPeiComptId(http_id, "method");
        pei_http_status_id = ProtPeiComptId(http_id, "status");
        pei_http_req_header_id = ProtPeiComptId(http_id, "req.header");
        pei_http_req_body_id = ProtPeiComptId(http_id, "req.body");
        pei_http_res_header_id = ProtPeiComptId(http_id, "res.header");
        pei_http_res_body_id = ProtPeiComptId(http_id, "res.body");
    }

    /* db connection */
    if (DBInit()) {
        printf("DB connections error\n");

        return -1;
    }
    pthread_mutex_init(&db_mux, NULL);

    return 0;
}


int DispEnd()
{
    return DBClose();
}


int DispInsPei(pei *ppei)
{
    int ret;

    LogPrintf(LV_DEBUG, "Xplico System PEI dispatcher");
    
    if (ppei != NULL) {
        if (ppei->prot_id == pop_id)
            ret = DispPop(ppei);
        else if (ppei->prot_id == smtp_id)
            ret = DispSmtp(ppei);
        else if (ppei->prot_id == imap_id)
            ret = DispImap(ppei);
        else if (ppei->prot_id == http_id)
            ret = DispHttp(ppei);
        else
            PeiPrint(ppei);

        /* free memory */
        if (ppei->ret == FALSE) {
            ProtStackFrmDisp(ppei->stack, TRUE);
            PeiFree(ppei);
        }
    }
    
    return 0;
}

