/* pei.h
 * Protocol Element Information.
 *
 *
 * Xplico - Internet Traffic Decoder
 * By Gianluca Costa <g.costa@iserm.com>
 * Copyright 2007 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef __PEI_H__
#define __PEI_H__

#include "istypes.h"
#include "packet.h"


typedef enum _eerror eerror;
enum _eerror {
    ELMT_ER_NONE = 0,
    ELMT_ER_HOLE,
    ELMT_ER_PARTIAL
};


typedef struct _pei_component pei_component;
struct _pei_component {
    short eid;        /**< identify univocaly an component type internal of PEI of paticular protocol */
    unsigned long id; /**< id of PEI component, used in case of PEI return to dissector */
    unsigned long time_cap;      /**< start capture time */
    unsigned long time_cap_end;  /**< capture time end   */
    char *strbuf;        /**< buffer string */
    char *name;          /**< file name */
    char *file_path;     /**< file path of component data */
    long file_size;      /**< file size */
    bool changed;        /**< true if data in the file is changed */
    eerror err;          /**< error type */
    pei_component *next; /**< next component */
};


typedef struct _pei pei;
struct _pei {
    int prot_id;                 /**< PEI of protocol whit this protocol id */
    unsigned long id;            /**< id (unic) of this pei */
    bool ret;                    /**< identify a PEI that return to dissector */ 
    pei_component *components;   /**< components list */
    unsigned long time_cap;      /**< start capture time */
    unsigned long time;          /**< decoding end time (auto assigned) */
    unsigned long serial;        /**< serial number (unique) used for order a pei (if necesary) */
    pstack_f *stack;             /**< pei protocol stack */
};


int PeiInit(pei *ppei);
int PeiIns(pei *ppei);
int PeiFree(pei *ppei);
void PeiPrint(pei *ppei);


#endif /* __PEI_H__ */
