/* tcp_garbage.c
 * Dissector to group together packet of tcp flow that haven't a specific dissector
 *
 *
 * Xplico - Internet Traffic Decoder
 * By Gianluca Costa <g.costa@iserm.com>
 * Copyright 2007 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <pcap.h>
#include <linux/tcp.h>


#include "proto.h"
#include "dmemory.h"
#include "etypes.h"
#include "flow.h"
#include "log.h"
#include "tcp_garbage.h"


static int ppp_id;
static int eth_id;
static int ip_id;
static int ipv6_id;
static int tcp_id;
static int ip_src_id;
static int ip_dst_id;
static int port_src_id;
static int port_dst_id;
static int lost_id;
static volatile int serial = 0;


static bool TcpGrbCheck(int flow_id)
{
    unsigned long pkt_num;

    pkt_num = FlowPktNum(flow_id);
    if (pkt_num >  TCP_GRB_PKT_LIMIT || (pkt_num > 0 && FlowIsClose(flow_id) == TRUE)) {
        return TRUE;
    }

    return FALSE;
}


packet* TcpGrbDissector(int flow_id)
{
    packet* pkt;
    const pstack_f *tcp, *ip;
    ftval port_p, ip_p, port_src, port_dst, ip_src, ip_dst, lost;
    struct in_addr ip_addr;
    int fd_pcap;
    char filename[256], ips_str[30], ipd_str[30];
    struct pcap_file_header fh;
    struct pcap_pkthdr pckt_header;
    unsigned long seq;
    unsigned long seq_s = 0;
    unsigned long seq_d = 0;
    bool ipv4;
    int count;

    LogPrintf(LV_DEBUG, "TCP garbage id: %d", flow_id);

    tcp = FlowStack(flow_id);
    ProtStackFrmDisp(tcp, TRUE);
    ip = ProtGetNxtFrame(tcp);
    ProtGetAttr(tcp, port_src_id, &port_src);
    ProtGetAttr(tcp, port_dst_id, &port_dst);
    ipv4 = FALSE;
    if (ProtFrameProtocol(ip) == ip_id)
        ipv4 = TRUE;
    if (ipv4) {
        ProtGetAttr(ip, ip_src_id, &ip_src);
        ProtGetAttr(ip, ip_dst_id, &ip_dst);
        ip_addr.s_addr = ip_src.uint32;
        LogPrintf(LV_DEBUG, "\tSRC: %s:%d", inet_ntoa(ip_addr), port_src.uint16);
        sprintf(ips_str, "%s.%d", inet_ntoa(ip_addr), port_src.uint16);
        ip_addr.s_addr = ip_dst.uint32;
        LogPrintf(LV_DEBUG, "\tDST: %s:%d", inet_ntoa(ip_addr), port_dst.uint16);
        sprintf(ipd_str, "%s.%d", inet_ntoa(ip_addr), port_dst.uint16);
    }
    else
        LogPrintf(LV_DEBUG, "TCP garbage IPv6");

    /* file pcap */
    //sprintf(filename, "%s/tcp_%d_grb_%s_%s.pcap", ProtTmpDir(), serial, ips_str, ipd_str);
    sprintf(filename, "/dev/null");
    serial++;
    fd_pcap = open(filename, O_WRONLY | O_CREAT, 0x01B6);
    memset(&fh, 0, sizeof(struct pcap_file_header));
    fh.magic = 0xA1B2C3D4;
    fh.version_major = PCAP_VERSION_MAJOR;
    fh.version_minor = PCAP_VERSION_MINOR;
    fh.snaplen = 65535;
    if (ProtGetNxtFrame(ip) != NULL) {
        if (ProtFrameProtocol(ProtGetNxtFrame(ip)) == eth_id)
            fh.linktype = DLT_EN10MB;
        else
            fh.linktype = DLT_RAW;
    }
    if (fd_pcap != -1)
        write(fd_pcap, (char *)&fh, sizeof(struct pcap_file_header));

    count = 0;
    pkt = FlowGetPkt(flow_id);
    while (pkt != NULL) {
        count++;
        //ProtStackFrmDisp(pkt->stk, TRUE);
        ProtGetAttr(pkt->stk, port_src_id, &port_p);
        ProtGetAttr(ProtGetNxtFrame(pkt->stk), ip_src_id, &ip_p);
        ProtGetAttr(tcp, port_src_id, &port_p);
        ProtGetAttr(ip, ip_src_id, &ip_p);
        ProtGetAttr(pkt->stk, lost_id, &lost);
        if (lost.uint8 == FALSE) {
#ifdef XPL_CHECK_CODE
            if (((pkt->raw + pkt->raw_len) < pkt->data)) {
                LogPrintf(LV_OOPS, "TCP data location error %p %p %lu %lu", pkt->raw, pkt->data, pkt->raw_len, pkt->len);
                ProtStackFrmDisp(pkt->stk, TRUE);
                exit(-1);
            }
            if ((pkt->data + pkt->len) > (pkt->raw + pkt->raw_len)) {
                LogPrintf(LV_OOPS, "TCP data dim error %p %p %lu %lu", pkt->raw, pkt->data, pkt->raw_len, pkt->len);
                ProtStackFrmDisp(pkt->stk, TRUE);
                exit(-1);
            }
#endif

#if 0
            if (pkt->len != 0) {
                if (ipv4) {
                    struct tcphdr *tcp;
                    tcp = (struct tcphdr *)(pkt->data-20);
                    seq = ntohl(tcp->seq);
                    if (FTCmp(&ip_src, &ip_p, FT_UINT32) == 0 && FTCmp(&port_p, &port_src, FT_UINT16) == 0) {
                        if (seq_s == 0)
                            seq_s = seq;
                        if (seq != seq_s) {
                            LogPrintf(LV_OOPS, "TCP src sequence error %lu/%lu 0x%x", seq, seq_s, *tcp);
                            ProtStackFrmDisp(pkt->stk, TRUE);
                            exit(-1);
                        }
                        seq_s = seq + pkt->len;
                    }
                    else {
                        if (seq_d == 0)
                            seq_d = seq;
                        if (seq != seq_d) {
                            LogPrintf(LV_OOPS, "TCP dst sequence error %lu/%lu", seq, seq_d);
                            ProtStackFrmDisp(pkt->stk, TRUE);
                            exit(-1);
                        }
                        seq_d = seq + pkt->len;
                    }
                }
                else {
                    LogPrintf(LV_FATAL, "Check for IPv6 to be implemented (%d)", __FUNCTION__);
                    exit(-1);
                }
            }
#endif
            pckt_header.caplen = pkt->raw_len;
            pckt_header.len = pkt->raw_len;
            pckt_header.ts.tv_sec = pkt->cap_sec;
            pckt_header.ts.tv_usec = pkt->cap_usec;
            if (fd_pcap != -1) {
                write(fd_pcap, (char *)&pckt_header, sizeof(struct pcap_pkthdr));
                write(fd_pcap, (char *)pkt->raw, pkt->raw_len);
            }
        }
        else {
#if 0
            if (FTCmp(&ip_src, &ip_p, FT_UINT32) == 0 && FTCmp(&port_p, &port_src, FT_UINT16) == 0) {
                if (seq_s != 0)
                    seq_s = seq_s + pkt->len;
            }
            else {
                if (seq_d != 0)
                    seq_d = seq_d + pkt->len;
            }
#endif
        }
        PktFree(pkt);
        pkt = FlowGetPkt(flow_id);
    }

    /* end */
    if (fd_pcap != -1)
        close(fd_pcap);

    FlowDelete(flow_id);

    LogPrintf(LV_DEBUG, "TCP garbage... bye bye  fid:%d count:%i", flow_id, count);

    return NULL;
}


int DissecRegist(void)
{
    proto_heury_dep hdep;

    memset(&hdep, 0, sizeof(proto_heury_dep));

    /* protocol name */
    ProtName("TCP garbage", "tcp-grb");

    /* dep: tcp */
    hdep.name = "tcp";
    hdep.ProtCheck = TcpGrbCheck;
    ProtHeuDep(&hdep);

    /* dissectors subdissectors registration */
    ProtDissectors(NULL, TcpGrbDissector, NULL, NULL);

    return 0;
}


int DissectInit(void)
{
    ppp_id = ProtId("ppp");
    eth_id = ProtId("eth");
    ip_id = ProtId("ip");
    ipv6_id = ProtId("ipv6");
    tcp_id = ProtId("tcp");
    ip_dst_id = ProtAttrId(ip_id, "ip.dst");
    ip_src_id = ProtAttrId(ip_id, "ip.src");
    port_dst_id = ProtAttrId(tcp_id, "tcp.dstport");
    port_src_id = ProtAttrId(tcp_id, "tcp.srcport");
    lost_id = ProtAttrId(tcp_id, "tcp.lost");

    return 0;
}
