/* session_decoding.c
 * Session decoding monitoring
 *
 *
 * Xplico System
 * By Gianluca Costa <g.costa@iserm.com>
 * Copyright 2007 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "session_decoding.h"


static char filename[DM_FILENAME_PATH];

static int ListSort(const void *a, const void *b)
{
    return strcmp(*(char **)a, *(char **)b);
}


int SeDeFind(char *main_dir, podec *tbl)
{
    DIR *dir, *dir1;
    struct dirent *entry, *entry1;
    int i, tmp, pol_id, sol_id;
    int new, next;
    int len_pol, len_ses;
    char dir_path[DM_FILENAME_PATH];

    dir = opendir(main_dir);
    if (dir == NULL) {
        perror("");
        exit(-1);
        return -1;
    }

    /* first empty position in tbl */
    next = 0;
    while (tbl[next].pol_id != -1)
        next++;

    /* pol directory */
    new = 0;
    len_pol = strlen(DM_POL_NAME);
    len_ses = strlen(DM_SESSION_NAME);
    while((entry = readdir(dir)) != NULL && new < DM_TBL_ADD) {
        if (entry->d_name[0] == '.')
            continue;
        /* check if this directory is a pol directory and if alrady exist in tbl */
        if (strncmp(entry->d_name, DM_POL_NAME, len_pol) == 0) {
            if (sscanf(entry->d_name, DM_POL_NAME"%i", &pol_id) == 1) {
                for (i=0; i<next; i++) {
                    if (tbl[i].pol_id == pol_id) {
                        break;
                    }
                }
                if (i == next) {
                    /* session directory */
                    sprintf(dir_path, DM_POL_DIR, main_dir, pol_id);
                    printf("%s\n", dir_path);
                    dir1 = opendir(dir_path);
                    if (dir1 != NULL) {
                        sol_id = INT_MAX;
                        while((entry1 = readdir(dir1)) != NULL) {
                            if (entry->d_name[0] == '.')
                                continue;
                            /* check if this directory is a pol directory and if alrady exist in tbl */
                            if (strncmp(entry1->d_name, DM_SESSION_NAME, len_ses) == 0) {
                                if (sscanf(entry1->d_name, DM_SESSION_NAME"%i", &tmp) == 1) {
                                    if (tmp < sol_id)
                                        sol_id = tmp;
                                }
                            }
                        }
                        closedir(dir1);
                        /* new pol */
                        tbl[next].pol_id = pol_id;
                        tbl[next].sol_id = sol_id;
                        tbl[next].run = FALSE;
                        memset(&tbl[next].pid, 0, sizeof(task));
                        tbl[next].end = FALSE;
                        tbl[next].name[0] = '\0';
                        tbl[next].size = 0;
                        tbl[next].filenum = 0;
                        next++;
                        new++;
                    }
                }
            }
        }
    }
    closedir(dir);

    return new;
}


int SeDeStart(char *main_dir, int pol, int session, task *pid)
{
    int ptsk;
    char app_path[DM_FILENAME_PATH];
    char config_file[DM_FILENAME_PATH];
    char work_dir[DM_FILENAME_PATH];
    char cmd[2*DM_FILENAME_PATH];

    /* start xplico */
    ptsk = fork();
    if (ptsk == 0) {

        /* create config file */
        sprintf(config_file, "%s/cfg/%s", main_dir, DM_XPLICO_CFG);
        sprintf(work_dir, DM_TMP_DIR, main_dir, pol);
        sprintf(cmd, "cp -a %s %s", config_file, work_dir);
        system(cmd);
        sprintf(config_file, DM_TMP_DIR"/%s", main_dir, pol, DM_XPLICO_CFG);
        sprintf(cmd, "echo LOG_DIR_PATH="DM_LOG_DIR" >> %s", main_dir, pol, config_file);
        system(cmd);
        sprintf(config_file, DM_TMP_DIR"/%s", main_dir, pol, DM_XPLICO_CFG);
        sprintf(cmd, "echo TMP_DIR_PATH="DM_TMP_DIR"/xplico >> %s", main_dir, pol, config_file);
        system(cmd);
        sprintf(cmd, "echo POL_ID=%i > "DM_DECOD_DIR"/%s", pol,
                main_dir, pol, session, POL_INIT_SESSION_FILE);
        system(cmd);
        sprintf(cmd, "echo SESSION_ID=%i >> "DM_DECOD_DIR"/%s", session,
                main_dir, pol, session, POL_INIT_SESSION_FILE);
        system(cmd);

        /* xplico processe */
        sprintf(app_path, "%s/bin/xplico", main_dir);
        sprintf(work_dir, DM_DECOD_DIR, main_dir, pol, session);
        execlp(app_path, "xplico", "-c" , config_file, "-m", "pol", "-d", work_dir, NULL);
        exit(-1);
    }
    else if (ptsk == -1) {
        return -1;
    }
    pid->xplico = ptsk;

    return 0;
}


int SeDeEnd(char *main_dir, int pol, int session, task *pid)
{
    char end_file[DM_FILENAME_PATH];
    int fd;

    /* end xplico */
    sprintf(end_file, DM_DECOD_DIR"/%s", main_dir, pol, session, POL_END_SESSION_FILE);
    fd = open(end_file, O_CREAT|O_RDWR, 0x01B6); /* only create file */
    if (fd != -1)
        close(fd);

    return 0;
}


char* SeDeFile(char *main_dir, int pol, int session)
{
    char newdir[DM_FILENAME_PATH];
    DIR *dir;
    struct dirent *entry;
    int i, num;
    char **list;
    
    sprintf(newdir, DM_NEW_DIR, main_dir, pol, session);
    
    dir = opendir(newdir);
    if (dir == NULL) {
        perror("");
        exit(-1);
        return NULL;
    }

    /* file list */
    num = 0;
    list = NULL;
    while((entry = readdir(dir)) != NULL) {
        if (entry->d_name[0] == '.')
            continue;
        list = realloc(list, sizeof(char *)*(num+1));
        list[num] = malloc(strlen(entry->d_name)+5);
        strcpy(list[num], entry->d_name);
        num++;
        
    }
    closedir(dir);
   
    /* sort */
    qsort(list, num, sizeof(char *), ListSort);
    if (num == 0) {
        return NULL;
    }
    if (list != NULL) {
        sprintf(filename, list[0]);
        for (i=0; i<num; i++) {
            free(list[i]);
        }
        free(list);
    }

    return filename;
}


int SeDeNextSession(char *main_dir, int pol, int session)
{
    DIR *dir;
    struct dirent *entry;
    char dir_path[DM_FILENAME_PATH];
    int len_pol, len_ses;
    int tmp;
    int sol_id;

    sol_id = -1;

    /* session directory */
    len_pol = strlen(DM_POL_NAME);
    len_ses = strlen(DM_SESSION_NAME);
    sprintf(dir_path, DM_POL_DIR, main_dir, pol);
    dir = opendir(dir_path);
    if (dir != NULL) {
        sol_id = INT_MAX;
        while((entry = readdir(dir)) != NULL) {
            if (entry->d_name[0] == '.')
                continue;
            /* check if this directory is a pol directory and if alrady exist in tbl */
            if (strncmp(entry->d_name, DM_SESSION_NAME, len_ses) == 0) {
                if (sscanf(entry->d_name, DM_SESSION_NAME"%i", &tmp) == 1) {
                    if (tmp < sol_id && tmp > session)
                        sol_id = tmp;
                }
            }
        }
        closedir(dir);
        
        /* check value */
        if (sol_id == INT_MAX) {
            sol_id = -1;
        }
    }
    
    return sol_id;
}


int SeDeRun(task *pid, pid_t chld)
{
    int app = sizeof(task)/sizeof(pid_t);

    /* check all application */
    if (pid->xplico == chld) {
        app--;
    }

    return app;
}


